@@ -1,4 +1,4 @@
-# $Id: crt.py 897 2014-04-28 20:41:26Z mwall $
+# $Id: crt.py 899 2014-05-03 12:30:06Z mwall $
 # Copyright 2013 Matthew Wall
 
 # Emit loop data to file in Cumulus realtime format.
@@ -18,15 +18,17 @@
 # are not directly provided by weewx in a LOOP packet.
 #
 # The cumulus 'specification' for realtime.txt is ambiguous:
-#   FIXME: should time be local or utc?
+#
+#   pressure trend interval is not specified, we use 3 hours for pressure
+#   temperature trend interval is not specified, we use 3 hours for temperature
+#   wind avg speed/dir interval is not specified, we use 10 minutes
+#   time zone is not specified, we use utc everywhere
+#   pressure is not specified, we use 'barometer' not 'pressure' or 'altimeter'
+#
+#   FIXME: what if None?  we return NULL, but cumulus spec does not specify
+#   FIXME: is_sunny is ill-defined.  what radiation threshold indicates true?
 #   FIXME: ensure day/month/hour/minute/second is prefixed with leading zero
-#   FIXME: what if None value?  return NULL, but cumulus spec cannot handle it
-#   FIXME: is_sunny is ill-defined (sun is always shining, but might be clouds)
-#   FIXME: should 'barometer' be pressure?  (cumulus is not specific)
-#   FIXME: what interval for pressure trend?
-#   FIXME: what interval should be used for average wind speed?
-#   FIXME: over what period should avg wind dir be calculated?
-#   FIXME: are max/min times local or utc?
+#   FIXME: max_rad is not yet implemented
 
 import math
 import time
@@ -103,31 +105,31 @@ def knotToBeaufort(x):
         return 11 # violent storm
     return 12 # hurricane
 
-def calcAvgWindSpeed(archive, ts):
-    sts = ts - 3600
+def calcAvgWindSpeed(archive, ts, interval=600):
+    sts = ts - interval
     val = archive.getSql("SELECT AVG(windSpeed) FROM archive "
-                         "WHERE dateTime>=? AND dateTime<=?",
+                         "WHERE dateTime>? AND dateTime<=?",
                          (sts, ts))
     return val[0]
 
-def calcAvgWindDir(archive, ts):
-    sts = ts - 3600
+def calcAvgWindDir(archive, ts, interval=600):
+    sts = ts - interval
     val = archive.getSql("SELECT AVG(windDir) FROM archive "
-                         "WHERE dateTime>=? AND dateTime<=?",
+                         "WHERE dateTime>? AND dateTime<=?",
                          (sts, ts))
     return val[0]
 
 def calc10MinHighGust(archive, ts):
     sts = ts - 600
     val = archive.getSql("SELECT MAX(windGust) FROM archive "
-                         "WHERE dateTime>=? AND dateTime<=?",
+                         "WHERE dateTime>? AND dateTime<=?",
                          (sts, ts))
     return val[0]
 
 def calc10MinAvgWindDir(archive, ts):
     sts = ts - 600
     val = archive.getSql("SELECT AVG(windGust) FROM archive "
-                         "WHERE dateTime>=? AND dateTime<=?",
+                         "WHERE dateTime>? AND dateTime<=?",
                          (sts, ts))
     return val[0]
 
@@ -136,36 +138,36 @@ def calcWindRun(archive, ts):
     run = 0
     sod_ts = weeutil.weeutil.startOfDay(ts)
     for row in archive.genSql("SELECT interval,windSpeed FROM archive "
-                              "WHERE dateTime>=? AND dateTime<=?", 
+                              "WHERE dateTime>? AND dateTime<=?", 
                               (sod_ts, ts)):
         run += row[1] * (row[0] / 60)
     return run
 
-# calculate trend over past n hours
+# calculate trend over past n hours, default to 3 hour window
 def calcTrend(label, new_val, archive, ts, n=3):
     lastts = ts - n * 3600
-    qstr = "SELECT %s FROM archive WHERE dateTime>=? AND dateTime<=?" % label
+    qstr = "SELECT %s FROM archive WHERE dateTime>? AND dateTime<=?" % label
     old_val = archive.getSql(qstr, (lastts, ts))
     return new_val - old_val[0]
 
 def calcRainHour(archive, ts):
     sts = ts - 3600
     val = archive.getSql("SELECT SUM(rain) FROM archive "
-                         "WHERE dateTime>=? AND dateTime<=?",
+                         "WHERE dateTime>? AND dateTime<=?",
                          (sts, ts))
     return val[0]
 
 def calcRainMonth(archive, ts):
     span = weeutil.weeutil.archiveMonthSpan(ts)
     val = archive.getSql("SELECT SUM(rain) FROM archive "
-                         "WHERE dateTime>=? AND dateTime<=?",
+                         "WHERE dateTime>? AND dateTime<=?",
                          (span.start, ts))
     return val[0]
 
 def calcRainYear(archive, ts):
     span = weeutil.weeutil.archiveYearSpan(ts)
     val = archive.getSql("SELECT SUM(rain) FROM archive "
-                         "WHERE dateTime>=? AND dateTime<=?",
+                         "WHERE dateTime>? AND dateTime<=?",
                          (span.start, ts))
     return val[0]
 
@@ -173,14 +175,14 @@ def calcRainYesterday(archive, ts):
     ts = weeutil.weeutil.startOfDay(ts)
     sts = ts - 3600*24;
     val = archive.getSql("SELECT SUM(rain) FROM archive "
-                         "WHERE dateTime>=? AND dateTime<=?",
+                         "WHERE dateTime>? AND dateTime<=?",
                          (sts, ts))
     return val[0]
 
 def calcETToday(archive, ts):
     sts = weeutil.weeutil.startOfDay(ts)
     val = archive.getSql("SELECT SUM(ET) FROM archive "
-                         "WHERE dateTime>=? AND dateTime<=?",
+                         "WHERE dateTime>? AND dateTime<=?",
                          (sts, ts))
     return val[0]
 
@@ -306,8 +308,10 @@ class CumulusRealTime(StdService):
         ut = weewx.units.getStandardUnitType(pu, 'rain')
         data['units_rain'] = UNITS_RAIN.get(ut[0],ut[0])
         data['wind_run'] = calcWindRun(archive, ts)
-        data['pressure_trend'] = calcTrend('pressure', packet.get('pressure'),
+        data['pressure_trend'] = calcTrend('barometer', packet.get('barometer'),
                                            archive, ts)
+        data['temperature_trend'] = calcTrend('outTemp', packet.get('outTemp'),
+                                              archive, ts)
         data['rain_month'] = calcRainMonth(archive, ts)
         data['rain_year'] = calcRainYear(archive, ts)
         data['rain_yesterday'] = calcRainYesterday(archive, ts)
@@ -323,10 +327,10 @@ class CumulusRealTime(StdService):
         v,t = calcMinMax('windGust', archive, ts, 'MAX')
         data['windGust_max'] = v
         data['windGust_max_time'] = t
-        v,t = calcMinMax('pressure', archive, ts, 'MAX')
+        v,t = calcMinMax('barometer', archive, ts, 'MAX')
         data['pressure_max'] = v
         data['pressure_max_time'] = t
-        v,t = calcMinMax('pressure', archive, ts, 'MIN')
+        v,t = calcMinMax('barometer', archive, ts, 'MIN')
         data['pressure_min'] = v
         data['pressure_min_time'] = t
         data['10min_high_gust'] = calc10MinHighGust(archive, ts)
@@ -382,9 +386,12 @@ class CumulusRealTime(StdService):
 
     def format(self, data, label):
         value = data.get(label, 'NULL')
+        if value is None:
+            value = 'NULL'
         return str(value)
 
     # the * indicates a field that is not part of a typical LOOP packet
+    # the ## indicates calculation is not yet implemented
     def create_realtime_string(self, data):
         fields = []
         datefmt = "%%d%s%%m%s%%y" % (self.datesep, self.datesep)
@@ -415,7 +422,7 @@ class CumulusRealTime(StdService):
         fields.append(self.format(data, 'inTemp'))                 # 23
         fields.append(self.format(data, 'inHumidity'))             # 24
         fields.append(self.format(data, 'windChill'))              # 25
-        fields.append(self.format(data, 'temperature_trend'))      # 26
+        fields.append(self.format(data, 'temperature_trend'))      # 26 *
         fields.append(self.format(data, 'outTemp_max'))            # 27 *
         fields.append(self.format(data, 'outTemp_max_time'))       # 28 *
         fields.append(self.format(data, 'outTemp_min'))            # 29 *
